@@ -33,14 +33,14 @@
 #'
 #' @param x numeric matrix.
 #' @param y numeric matrix for the second group of observations.
+#' @param null true values of the means for the null hypothesis.
+#' A single number or numeric vector with values for each observation.
 #' @param alternative alternative hypothesis to use for each row/column of x.
 #' A single string or a vector with values for each observation.
 #' Values must be one of "two.sided" (default), "greater" or "less".
-#' @param mu true values of the means for the null hypothesis.
-#' A single number or numeric vector with values for each observation.
 #' @param conf.level confidence levels used for the confidence intervals.
 #' A single number or a numeric vector with values for each observation.
-#' All values must be in the range of [0:1].
+#' All values must be in the range of [0:1] or NA.
 #'
 #' @return a data.frame where each row contains the results of a t.test
 #' performed on the corresponding row/column of x.
@@ -63,10 +63,14 @@
 #' 15. pvalue - p-value\cr
 #' 16. conf.low - lower bound of the confidence interval\cr
 #' 17. conf.high - higher bound of the confidence interval\cr
-#' 18. alternative - chosen alternative hypothesis\cr
-#' 19. mean.null - mean of the null hypothesis\cr
+#' 18. mean.null - mean of the null hypothesis\cr
+#' 19. alternative - chosen alternative hypothesis\cr
 #' 20. conf.level - chosen confidence level
 #'
+#' @note
+#' For a marked increase in computation speed turn off the calculation of
+#' confidence interval by setting \code{conf.level} to NA.
+#'
 #' @seealso \code{t.test()}
 #'
 #' @examples
@@ -80,9 +84,9 @@
 #' @author Karolis Koncevičius
 #' @name ttest
 #' @export
-row_t_equalvar <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  force(x)
-  force(y)
+row_t_equalvar <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  is.null(x)
+  is.null(y)
 
   if(is.vector(x))
     x <- matrix(x, nrow=1)
@@ -104,84 +108,91 @@ row_t_equalvar <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95)
   assert_equal_nrow(x, y)
 
   if(length(alternative)==1)
-    alternative <- rep(alternative, length.out=nrow(x))
+    alternative <- rep.int(alternative, nrow(x))
   assert_character_vec_length(alternative, 1, nrow(x))
 
   choices <- c("two.sided", "less", "greater")
   alternative <- choices[pmatch(alternative, choices, duplicates.ok=TRUE)]
   assert_all_in_set(alternative, choices)
 
-  if(length(mu)==1)
-    mu <- rep(mu, length.out=nrow(x))
-  assert_numeric_vec_length(mu, 1, nrow(x))
-  assert_all_in_closed_interval(mu, -Inf, Inf)
+  if(length(null)==1)
+    null <- rep.int(null, nrow(x))
+  assert_numeric_vec_length(null, 1, nrow(x))
+  assert_all_in_closed_interval(null, -Inf, Inf)
 
+  if(all(is.na(conf.level)))
+    conf.level[] <- NA_real_
   if(length(conf.level)==1)
-    conf.level <- rep(conf.level, length.out=nrow(x))
+    conf.level <- rep.int(conf.level, nrow(x))
   assert_numeric_vec_length(conf.level, 1, nrow(x))
-  assert_all_in_closed_interval(conf.level, 0, 1)
+  assert_all_in_closed_interval(conf.level, 0, 1, na.allow=TRUE)
 
 
   mxs  <- rowMeans(x, na.rm=TRUE)
   mys  <- rowMeans(y, na.rm=TRUE)
   mxys <- mxs - mys
 
-  nxs  <- rep.int(ncol(x), nrow(x)) - matrixStats::rowCounts(is.na(x))
-  nys  <- rep.int(ncol(y), nrow(y)) - matrixStats::rowCounts(is.na(y))
+  nxs  <- ncol(x) - matrixStats::rowCounts(x, value=NA)
+  nys  <- ncol(y) - matrixStats::rowCounts(y, value=NA)
   nxys <- nxs + nys
 
-  vxs <- rowSums((x-mxs)^2, na.rm=TRUE) / (nxs-1)
-  vys <- rowSums((y-mys)^2, na.rm=TRUE) / (nys-1)
+  vxs <- rowVars(x, n=nxs, m=mxs, na.rm=TRUE)
+  vys <- rowVars(y, n=nys, m=mys, na.rm=TRUE)
 
-  dfs <- nxs + nys - 2
+  dfs  <- nxs + nys - 2
 
-  vs <- rep(0, nrow(x))
-  vs <- ifelse(nxs > 1, vs + (nxs-1) * vxs, vs)
-  vs <- ifelse(nys > 1, vs + (nys-1) * vys, vs)
+  vs <- rep.int(0, nrow(x))
+  inds <- nxs > 1
+  vs[inds] <- vs[inds] + (nxs[inds]-1) * vxs[inds]
+  inds <- nys > 1
+  vs[inds] <- vs[inds] + (nys[inds]-1) * vys[inds]
   vs <- vs/dfs
+
   stders <- sqrt(vs * (1/nxs + 1/nys))
 
-  tres <- do_ttest(mxys, mu, stders, alternative, dfs, conf.level)
+  tres <- do_ttest(mxys, null, stders, alternative, dfs, conf.level)
 
 
   w1 <- nxys < 3
-  showWarning(w1, 'had less than 3 total observations')
+  showWarning(w1, 't_equalvar', 'had less than 3 total observations')
 
   w2 <- !w1 & nxs < 1
-  showWarning(w2, 'had zero "x" observations')
+  showWarning(w2, 't_equalvar', 'had zero "x" observations')
 
   w3 <- !w1 & nys < 1
-  showWarning(w3, 'had zero "y" observations')
+  showWarning(w3, 't_equalvar', 'had zero "y" observations')
 
   w4 <- stders <= 10 * .Machine$double.eps * pmax(abs(mxs), abs(mys))
-  showWarning(w4, 'were essentially constant')
+  showWarning(w4, 't_equalvar', 'had essentially constant values')
 
 
-  tres[w1 | w2 | w3 | w4,] <- NA
+  stders[w1 | w2 | w3 | w4] <- NA
+  dfs[w1 | w2 | w3 | w4]    <- NA
+  tres[w1 | w2 | w3 | w4,]  <- NA
 
   rnames <- rownames(x)
   if(!is.null(rnames)) rnames <- make.unique(rnames)
   data.frame(obs.x=nxs, obs.y=nys, obs.tot=nxys, mean.x=mxs, mean.y=mys,
              mean.diff=mxys, var.x=vxs, var.y=vys, var.pooled=vs,
              stderr=stders, df=dfs, statistic=tres[,1], pvalue=tres[,2],
-             conf.low=tres[,3], conf.high=tres[,4], alternative=alternative,
-             mean.null=mu, conf.level=conf.level, stringsAsFactors=FALSE,
-             row.names=rnames
+             conf.low=tres[,3], conf.high=tres[,4], mean.null=null,
+             alternative=alternative, conf.level=conf.level,
+             stringsAsFactors=FALSE, row.names=rnames
              )
 }
 
 #' @rdname ttest
 #' @export
-col_t_equalvar <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  row_t_equalvar(t(x), t(y), alternative=alternative, mu=mu, conf.level=conf.level)
+col_t_equalvar <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  row_t_equalvar(t(x), t(y), null=null, alternative=alternative, conf.level=conf.level)
 }
 
 
 #' @rdname ttest
 #' @export
-row_t_welch <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  force(x)
-  force(y)
+row_t_welch <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  is.null(x)
+  is.null(y)
 
   if(is.vector(x))
     x <- matrix(x, nrow=1)
@@ -203,34 +214,36 @@ row_t_welch <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
   assert_equal_nrow(x, y)
 
   if(length(alternative)==1)
-    alternative <- rep(alternative, length.out=nrow(x))
+    alternative <- rep.int(alternative, nrow(x))
   assert_character_vec_length(alternative, 1, nrow(x))
 
   choices <- c("two.sided", "less", "greater")
   alternative <- choices[pmatch(alternative, choices, duplicates.ok=TRUE)]
   assert_all_in_set(alternative, choices)
 
-  if(length(mu)==1)
-    mu <- rep(mu, length.out=nrow(x))
-  assert_numeric_vec_length(mu, 1, nrow(x))
-  assert_all_in_closed_interval(mu, -Inf, Inf)
+  if(length(null)==1)
+    null <- rep.int(null, nrow(x))
+  assert_numeric_vec_length(null, 1, nrow(x))
+  assert_all_in_closed_interval(null, -Inf, Inf)
 
+  if(all(is.na(conf.level)))
+    conf.level[] <- NA_real_
   if(length(conf.level)==1)
-    conf.level <- rep(conf.level, length.out=nrow(x))
+    conf.level <- rep.int(conf.level, nrow(x))
   assert_numeric_vec_length(conf.level, 1, nrow(x))
-  assert_all_in_closed_interval(conf.level, 0, 1)
+  assert_all_in_closed_interval(conf.level, 0, 1, na.allow=TRUE)
 
 
   mxs  <- rowMeans(x, na.rm=TRUE)
   mys  <- rowMeans(y, na.rm=TRUE)
   mxys <- mxs - mys
 
-  nxs  <- rep.int(ncol(x), nrow(x)) - matrixStats::rowCounts(is.na(x))
-  nys  <- rep.int(ncol(y), nrow(y)) - matrixStats::rowCounts(is.na(y))
+  nxs  <- ncol(x) - matrixStats::rowCounts(x, value=NA)
+  nys  <- ncol(y) - matrixStats::rowCounts(y, value=NA)
   nxys <- nxs + nys
 
-  vxs <- rowSums((x-mxs)^2, na.rm=TRUE) / (nxs-1)
-  vys <- rowSums((y-mys)^2, na.rm=TRUE) / (nys-1)
+  vxs <- rowVars(x, n=nxs, m=mxs, na.rm=TRUE)
+  vys <- rowVars(y, n=nys, m=mys, na.rm=TRUE)
 
   stderxs <- vxs/nxs
   stderys <- vys/nys
@@ -238,42 +251,43 @@ row_t_welch <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
   dfs     <- stders*stders / (stderxs*stderxs/(nxs - 1) + stderys*stderys/(nys - 1))
   stders  <- sqrt(stders)
 
-  tres <- do_ttest(mxys, mu, stders, alternative, dfs, conf.level)
+  tres <- do_ttest(mxys, null, stders, alternative, dfs, conf.level)
 
 
   w1 <- nxs < 2
-  showWarning(w1, 'had less than 2 "x" observations')
+  showWarning(w1, 't_welch', 'had less than 2 "x" observations')
 
   w2 <- !w1 & nys < 2
-  showWarning(w2, 'had less than 2 "y" observations')
+  showWarning(w2, 't_welch', 'had less than 2 "y" observations')
 
   w3 <- stders <= 10 * .Machine$double.eps * pmax(abs(mxs), abs(mys))
-  showWarning(w3, 'were essentially constant')
+  showWarning(w3, 't_welch', 'had essentially constant values')
 
-  tres[w1 | w2 | w3,] <- NA
+  stders[w1 | w2 | w3] <- NA
+  dfs[w1 | w2 | w3]     <- NA
+  tres[w1 | w2 | w3,]   <- NA
 
   rnames <- rownames(x)
   if(!is.null(rnames)) rnames <- make.unique(rnames)
   data.frame(obs.x=nxs, obs.y=nys, obs.tot=nxys, mean.x=mxs, mean.y=mys,
              mean.diff=mxys, var.x=vxs, var.y=vys, stderr=stders, df=dfs,
              statistic=tres[,1], pvalue=tres[,2], conf.low=tres[,3],
-             conf.high=tres[,4], alternative=alternative, mean.null=mu,
-             conf.level=conf.level, stringsAsFactors=FALSE,
-             row.names=rnames
+             conf.high=tres[,4], mean.null=null, alternative=alternative,
+             conf.level=conf.level, stringsAsFactors=FALSE, row.names=rnames
              )
 }
 
 #' @rdname ttest
 #' @export
-col_t_welch <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  row_t_welch(t(x), t(y), alternative=alternative, mu=mu, conf.level=conf.level)
+col_t_welch <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  row_t_welch(t(x), t(y), null=null, alternative=alternative, conf.level=conf.level)
 }
 
 
 #' @rdname ttest
 #' @export
-row_t_onesample <- function(x, alternative="two.sided", mu=0, conf.level=0.95) {
-  force(x)
+row_t_onesample <- function(x, null=0, alternative="two.sided", conf.level=0.95) {
+  is.null(x)
 
   if(is.vector(x))
     x <- matrix(x, nrow=1)
@@ -285,66 +299,69 @@ row_t_onesample <- function(x, alternative="two.sided", mu=0, conf.level=0.95) {
 
 
   if(length(alternative)==1)
-    alternative <- rep(alternative, length.out=nrow(x))
+    alternative <- rep.int(alternative, nrow(x))
   assert_character_vec_length(alternative, 1, nrow(x))
 
   choices <- c("two.sided", "less", "greater")
   alternative <- choices[pmatch(alternative, choices, duplicates.ok=TRUE)]
   assert_all_in_set(alternative, choices)
 
-  if(length(mu)==1)
-    mu <- rep(mu, length.out=nrow(x))
-  assert_numeric_vec_length(mu, 1, nrow(x))
-  assert_all_in_closed_interval(mu, -Inf, Inf)
+  if(length(null)==1)
+    null <- rep.int(null, nrow(x))
+  assert_numeric_vec_length(null, 1, nrow(x))
+  assert_all_in_closed_interval(null, -Inf, Inf)
 
+  if(all(is.na(conf.level)))
+    conf.level[] <- NA_real_
   if(length(conf.level)==1)
-    conf.level <- rep(conf.level, length.out=nrow(x))
+    conf.level <- rep.int(conf.level, nrow(x))
   assert_numeric_vec_length(conf.level, 1, nrow(x))
-  assert_all_in_closed_interval(conf.level, 0, 1)
+  assert_all_in_closed_interval(conf.level, 0, 1, na.allow=TRUE)
 
 
-  nxs <- rep.int(ncol(x), nrow(x)) - matrixStats::rowCounts(is.na(x))
+  nxs <- ncol(x) - matrixStats::rowCounts(x, value=NA)
   mxs <- rowMeans(x, na.rm=TRUE)
-  vxs <- rowSums((x-mxs)^2, na.rm=TRUE) / (nxs-1)
-  dfs <- nxs-1
+  vxs <- rowVars(x, n=nxs, m=mxs, na.rm=TRUE)
+
   stders <- sqrt(vxs/nxs)
+  dfs <- nxs-1
 
-
-  tres <- do_ttest(mxs, mu, stders, alternative, dfs, conf.level)
+  tres <- do_ttest(mxs, null, stders, alternative, dfs, conf.level)
 
 
   w1 <- nxs < 2
-  showWarning(w1, 'had less than 2 "x" observations')
+  showWarning(w1, 't_onesample', 'had less than 2 "x" observations')
 
   w2 <- !w1 & stders <= 10 * .Machine$double.eps * abs(mxs)
-  showWarning(w2, 'were essentially constant')
+  showWarning(w2, 't_onesample', 'had essentially constant values')
 
 
-  tres[w1 | w2,] <- NA
+  stders[w1 | w2]  <- NA
+  dfs[w1 | w2]     <- NA
+  tres[w1 | w2,]   <- NA
 
 
   rnames <- rownames(x)
   if(!is.null(rnames)) rnames <- make.unique(rnames)
   data.frame(obs=nxs, mean=mxs, var=vxs, stderr=stders, df=dfs,
              statistic=tres[,1], pvalue=tres[,2], conf.low=tres[,3],
-             conf.high=tres[,4], alternative=alternative, mean.null=mu,
-             conf.level=conf.level, stringsAsFactors=FALSE,
-             row.names=rnames
+             conf.high=tres[,4], mean.null=null, alternative=alternative,
+             conf.level=conf.level, stringsAsFactors=FALSE, row.names=rnames
              )
 }
 
 #' @rdname ttest
 #' @export
-col_t_onesample <- function(x, alternative="two.sided", mu=0, conf.level=0.95) {
-  row_t_onesample(t(x), alternative=alternative, mu=mu, conf.level=conf.level)
+col_t_onesample <- function(x, null=0, alternative="two.sided", conf.level=0.95) {
+  row_t_onesample(t(x), null=null, alternative=alternative, conf.level=conf.level)
 }
 
 
 #' @rdname ttest
 #' @export
-row_t_paired <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  force(x)
-  force(y)
+row_t_paired <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  is.null(x)
+  is.null(y)
 
   if(is.vector(x))
     x <- matrix(x, nrow=1)
@@ -367,22 +384,24 @@ row_t_paired <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
   assert_equal_ncol(x, y)
 
   if(length(alternative)==1)
-    alternative <- rep(alternative, length.out=nrow(x))
+    alternative <- rep.int(alternative, nrow(x))
   assert_character_vec_length(alternative, 1, nrow(x))
 
   choices <- c("two.sided", "less", "greater")
   alternative <- choices[pmatch(alternative, choices, duplicates.ok=TRUE)]
   assert_all_in_set(alternative, choices)
 
-  if(length(mu)==1)
-    mu <- rep(mu, length.out=nrow(x))
-  assert_numeric_vec_length(mu, 1, nrow(x))
-  assert_all_in_closed_interval(mu, -Inf, Inf)
+  if(length(null)==1)
+    null <- rep.int(null, nrow(x))
+  assert_numeric_vec_length(null, 1, nrow(x))
+  assert_all_in_closed_interval(null, -Inf, Inf)
 
+  if(all(is.na(conf.level)))
+    conf.level[] <- NA_real_
   if(length(conf.level)==1)
-    conf.level <- rep(conf.level, length.out=nrow(x))
+    conf.level <- rep.int(conf.level, nrow(x))
   assert_numeric_vec_length(conf.level, 1, nrow(x))
-  assert_all_in_closed_interval(conf.level, 0, 1)
+  assert_all_in_closed_interval(conf.level, 0, 1, na.allow=TRUE)
 
 
   xy <- x-y
@@ -391,44 +410,43 @@ row_t_paired <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
   mys  <- rowMeans(y, na.rm=TRUE)
   mxys <- rowMeans(xy, na.rm=TRUE)
 
-  nxs  <- rep.int(ncol(x), nrow(x)) - matrixStats::rowCounts(is.na(x))
-  nys  <- rep.int(ncol(y), nrow(y)) - matrixStats::rowCounts(is.na(y))
-  nxys <- rep.int(ncol(xy), nrow(xy)) - matrixStats::rowCounts(is.na(xy))
+  nxs  <- ncol(x)  - matrixStats::rowCounts(x, value=NA)
+  nys  <- ncol(y)  - matrixStats::rowCounts(y, value=NA)
+  nxys <- ncol(xy) - matrixStats::rowCounts(xy, value=NA)
 
-  vxs <- rowSums((x-mxs)^2, na.rm=TRUE) / (nxs-1)
-  vxs[nxs < 2] <- NA
-  vys <- rowSums((y-mys)^2, na.rm=TRUE) / (nys-1)
-  vys[nys < 2] <- NA
-
-  vxys <- rowSums((xy-mxys)^2, na.rm=TRUE) / (nxys-1)
+  vxs  <- rowVars(x, n=nxs, m=mxs, na.rm=TRUE)
+  vys  <- rowVars(y, n=nys, m=mys, na.rm=TRUE)
+  vxys <- rowVars(xy, n=nxys, m=mxys, na.rm=TRUE)
 
   stders <- sqrt(vxys/nxys)
   dfs <- nxys-1
 
-  tres <- do_ttest(mxys, mu, stders, alternative, dfs, conf.level)
+  tres <- do_ttest(mxys, null, stders, alternative, dfs, conf.level)
 
 
   w1 <- nxys < 2
-  showWarning(w1, 'had less than 2 paired observations')
+  showWarning(w1, 't_paired', 'had less than 2 paired observations')
 
   w2 <- stders <= 10 * .Machine$double.eps * abs(mxys)
-  showWarning(w2, 'were essentially constant')
+  showWarning(w2, 't_paired', 'had essentially constant values')
 
-  tres[w1 | w2,] <- NA
+  stders[w1 | w2] <- NA
+  dfs[w1 | w2]    <- NA
+  tres[w1 | w2,]  <- NA
 
   rnames <- rownames(x)
   if(!is.null(rnames)) rnames <- make.unique(rnames)
   data.frame(obs.x=nxs, obs.y=nys, obs.paired=nxys, mean.x=mxs, mean.y=mys,
              mean.diff=mxys, var.x=vxs, var.y=vys, var.diff=vxys,
              stderr=stders, df=dfs, statistic=tres[,1], pvalue=tres[,2],
-             conf.low=tres[,3], conf.high=tres[,4], alternative=alternative,
-             mean.null=mu, conf.level=conf.level, stringsAsFactors=FALSE,
-             row.names=rnames
+             conf.low=tres[,3], conf.high=tres[,4], mean.null=null,
+             alternative=alternative, conf.level=conf.level,
+             stringsAsFactors=FALSE, row.names=rnames
              )
 }
 
 #' @rdname ttest
 #' @export
-col_t_paired <- function(x, y, alternative="two.sided", mu=0, conf.level=0.95) {
-  row_t_paired(t(x), t(y), alternative=alternative, mu=mu, conf.level=conf.level)
+col_t_paired <- function(x, y, null=0, alternative="two.sided", conf.level=0.95) {
+  row_t_paired(t(x), t(y), null=null, alternative=alternative, conf.level=conf.level)
 }
